/*
 *        spRwPluginP.h
 */

#ifndef __SPRWPLUGINP_H
#define __SPRWPLUGINP_H

#include <sp/spRwPlugin.h>
#include <sp/spOutputPluginP.h>

#ifdef __cplusplus
extern "C" {
#endif

struct _spRwPluginRec
{
    void *host_data;
    void *user_data;
    
    spPluginType type;
    const char *plugin_id;
    long version_id;
    int priority;
    unsigned long caps;
    spBool (*init)(const char *lang);
    spBool (*free)(void);
    const char *description;
    const char *information;
    
    void *(*init_instance)(const char *lang);
    spBool (*free_instance)(void *instance);
    spOptions (*init_options)(void *instance, const char *lang);
    spBool (*free_options)(void *instance, spOptions options);
    
    spPluginDeviceType device_type;
    
    const char **file_type_list;
    const char **file_desc_list;
    const char **file_filter_list;
    spBool (*is_supported)(const char *filename); /* specify suffix part such as ".wav" for write mode */
    spBool (*set_file_type)(void *instance, int index);
    int (*get_file_type)(void *instance);
    unsigned long song_info_mask;
    spBool (*set_song_info)(void *instance, spSongInfo *song_info);
    spBool (*get_song_info)(void *instance, spSongInfo *song_info);
    const char *(*get_best_suffix)(void *instance);
    
    spBool (*get_num_device)(void *instance, int *num_device);
    const char *(*get_device_name)(void *instance, int index);
    spBool (*select_device)(void *instance, int index);
    spBool (*set_buffer_size)(void *instance, int buffer_size);
    spBool (*get_buffer_size)(void *instance, int *buffer_size);
    spBool (*set_volume)(void *instance, int channel, int volume);
    spBool (*get_volume)(void *instance, int channel, int *volume);
    
    spBool (*set_sample_bit)(void *instance, int samp_bit);
    spBool (*get_sample_bit)(void *instance, int *samp_bit);
    spBool (*set_channel)(void *instance, int num_channel);
    spBool (*get_channel)(void *instance, int *num_channel);
    spBool (*set_sample_rate)(void *instance, double samp_rate);
    spBool (*get_sample_rate)(void *instance, double *samp_rate);
    spBool (*set_other_info)(void *instance, const char *id, char *data);
    spBool (*get_other_info)(void *instance, const char *id, char *data);

    spPluginError (*open)(void *instance, const char *filename, const char *mode);
    spBool (*close)(void *instance);
    spPluginState (*get_state)(void *instance);
    spLong (*get_current_pos)(void *instance);
    spBool (*stop)(void *instance);
    spBool (*pause)(void *instance);
    spBool (*restart)(void *instance);
    
    long (*write)(void *instance, char *data, long length);
    spBool (*flush)(void *instance);
    spBool (*set_total_length)(void *instance, spLong total_length);
    
    long (*read)(void *instance, char *data, long length);
    spBool (*seek)(void *instance, spLong pos);
    spLong (*get_total_length)(void *instance);
};

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPRWPLUGINP_H */
