/*
 *        spWave_Wav.h
 */

#ifndef __SPWAVE_WAV_H
#define __SPWAVE_WAV_H

#if defined(_WIN32)
#include <windows.h>
#if !defined(__CYGWIN32__)
#include <mmreg.h>
#endif
#endif

#include <sp/spWave.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef WAVE_FORMAT_PCM
#define        WAVE_FORMAT_PCM                        (0x0001)
#endif
    
#ifndef WAVE_FORMAT_UNKNOWN
#define        WAVE_FORMAT_UNKNOWN                (0x0000)
#define        WAVE_FORMAT_ADPCM                (0x0002)
#define WAVE_FORMAT_IEEE_FLOAT                (0x0003)
#define        WAVE_FORMAT_ALAW                (0x0006)
#define        WAVE_FORMAT_MULAW                (0x0007)
#define        WAVE_FORMAT_OKI_ADPCM                (0x0010)
#define        WAVE_FORMAT_DIGISTD                (0x0015)
#define        WAVE_FORMAT_DIGIFIX                (0x0016)
#endif /* !WAVE_FORMAT_UNKNOWN */

#ifndef AVIF_HASINDEX
#define AVIF_HASINDEX       0x00000010
#define AVIF_MUSTUSEINDEX   0x00000020 
#define AVIF_ISINTERLEAVED  0x00000100 
#define AVIF_TRUSTCKTYPE    0x00000800
#define AVIF_WASCAPTUREFILE 0x00010000 
#define AVIF_COPYRIGHTED    0x00020000
#endif
    
#ifndef AVISF_DISABLED
#define AVISF_DISABLED   0x00000001 
#define AVISF_VIDEO_PALCHANGES  0x00010000 
#define AVIIF_LIST       0x00000001 
#define AVIIF_KEYFRAME   0x00000010 
#define AVIIF_NO_TIME    0x00000100 
#define AVIIF_COMPRESSOR 0x0FFF0000
#endif
    
#ifndef AVI_INDEX_OF_INDEXES
#define AVI_INDEX_OF_INDEXES      0x00 
#define AVI_INDEX_OF_CHUNKS       0x01 
#define AVI_INDEX_OF_TIMED_CHUNKS 0x02 
#define AVI_INDEX_OF_SUB_2FIELD   0x03 
#define AVI_INDEX_IS_DATA         0x80 
#define AVI_INDEX_SUB_DEFAULT     0x00
#define AVI_INDEX_SUB_2FIELD      0x01    
#endif
    
#if !defined(_WIN32)
typedef unsigned long DWORD; /* internal format (64bit unsigned long is OK) */
typedef unsigned short WORD;
typedef unsigned char BYTE;
typedef long LONG;           /* internal format (64bit long is OK) */

typedef struct _SMALL_RECT {
    short Left;
    short Top;
    short Right;
    short Bottom;
} SMALL_RECT;

typedef struct { 
    WORD  wFormatTag; 
    WORD  nChannels; 
    DWORD nSamplesPerSec; 
    DWORD nAvgBytesPerSec; 
    WORD  nBlockAlign; 
    WORD  wBitsPerSample; 
    WORD  cbSize; 
} WAVEFORMATEX; 

typedef struct tagBITMAPINFOHEADER {
    DWORD  biSize; 
    LONG   biWidth; 
    LONG   biHeight; 
    WORD   biPlanes; 
    WORD   biBitCount; 
    DWORD  biCompression; 
    DWORD  biSizeImage; 
    LONG   biXPelsPerMeter; 
    LONG   biYPelsPerMeter; 
    DWORD  biClrUsed; 
    DWORD  biClrImportant; 
} BITMAPINFOHEADER, *PBITMAPINFOHEADER; 
#endif

/*
 * Following parameters must be specified.
 *        wFormatTag
 *         nChannels
 *         nSamplesPerSec
 *         wBitsPerSample
 */
typedef  struct {
    char RiffID[4];             /* 'R', 'I', 'F', 'F' */
    DWORD RiffSize;             /* (file size - 8) [bytes] */
    char WaveID[4];             /* 'W', 'A', 'V', 'E' */
    char FmtID[4];              /* 'f', 'm', 't', ' ' */
    DWORD FmtSize;              /* format chunk size [bytes] */
    WORD wFormatTag;            /* 1: WAVE_FORMAT_PCM, 0x0000: WAVE_FORMAT_UNKNOWN ... */
    WORD nChannels;             /* number of channels */
    DWORD nSamplesPerSec;       /* sampling frequency [Hz] */
    DWORD nAvgBytesPerSec;      /* nSamplesPerSec * nBlockAlign */
    WORD nBlockAlign;           /* (nChannels * wBitsPerSample) / 8 [bytes] */
    WORD wBitsPerSample;        /* 16 or 8 [bits] */
    char DataID[4];             /* 'd' 'a' 't' 'a' */
    DWORD nDataBytes;           /* data chunk size [bytes] */
} WAVEHEADER;

typedef struct _DVInfo {
    DWORD dwDVAAuxSrc;
    DWORD dwDVAAuxCtl;
    DWORD dwDVAAuxSrc1;
    DWORD dwDVAAuxCtl1;
    DWORD dwDVVAuxSrc;
    DWORD dwDVVAuxCtl;
    DWORD dwDVReserved[2];
} DVInfo;

typedef struct {
    char ListID[4];             /* 'L', 'I', 'S', 'T' */
    DWORD ListSize;
    char fcc[4];
} AviListHeader;

typedef struct {
    char RiffID[4];             /* 'R', 'I', 'F', 'F' */
    DWORD RiffSize;             /* (file size - 8) [bytes] */
    char AviID[4];              /* 'A', 'V', 'I', ' ' */
    AviListHeader hdrl;         /* hdrl */
    char avihID[4];             /* 'a', 'v', 'i', 'h' */
    DWORD avihSize;
    DWORD dwMicroSecPerFrame;
    DWORD dwMaxBytesPerSec;
    DWORD dwPaddingGranularity;
    DWORD dwFlags;
    DWORD dwTotalFrames;
    DWORD dwInitialFrames;
    DWORD dwStreams;
    DWORD dwSuggestedBufferSize;
    DWORD dwWidth;
    DWORD dwHeight;
    DWORD dwReserved[4];
} AviMainHeader;

typedef struct {
    AviListHeader strl;         /* strl */
    char strhID[4];             /* 's', 't', 'r', 'h' */
    DWORD strhSize;
    char fccType[4];            /* video:'vids' or 'iavs', audio:'auds', subtitle:'txts' */
    char fccHandler[4];         /* codec fourcc */
    DWORD dwFlags;
    WORD wPriority;
    WORD wLanguage;
    DWORD dwInitialFrames;
    DWORD dwScale;
    DWORD dwRate;
    DWORD dwStart;
    DWORD dwLength;
    DWORD dwSuggestedBufferSize;
    DWORD dwQuality;
    DWORD dwSampleSize;
    SMALL_RECT rcFrame;
    
    char strfID[4];             /* 's', 't', 'r', 'f' */
    DWORD strfSize;
} AviStreamHeader;

typedef struct _AviExtHeader {
    AviListHeader odml;         /* odml */
    char dmlhID[4];
    DWORD dmlhSize;
    DWORD dwGrandFrames;
    DWORD dwFuture[61];
} AviExtHeader;

typedef struct _AviOldIndexEntry {
    char chunkID[4];
    DWORD dwFlags;
    DWORD dwChunkOffset;
    DWORD dwChunkLength;
} AviOldIndexEntry;

typedef struct _AviOldIndexChunk {
    char idx1ID[4];             /* 'idx1' */
    DWORD idx1Size;
    AviOldIndexEntry *aIndex;
} AviOldIndexChunk;
    
typedef struct _AviStdIndexEntry {
    DWORD dwOffset;
    DWORD dwSize;
} AviStdIndexEntry;

typedef struct _AviSuperIndexEntry {
    char qwOffset[8];
    DWORD dwSize;
    DWORD dwDuration;
} AviSuperIndexEntry;

typedef struct _AviIndexChunkHeader {
    char fcc[4];                /* 'indx' or 'ix??' */
    DWORD dwSize;
    WORD wLongsPerEntry;        /* single index size = 4 * wLongsPerEntry */
    BYTE bIndexSubType;
    BYTE bIndexType;
    DWORD nEntriesInUse;        /* number of indices */
    char chunkID[4];
    char qwOffset[8];
    DWORD dwReserved;
} AviIndexChunkHeader;
    
typedef struct _AviIndexChunk {
    AviIndexChunkHeader header;
    char *aIndex;
} AviIndexChunk;
    
typedef struct _AviSuperIndexChunk {
    AviIndexChunkHeader header;
    AviSuperIndexEntry *aIndex;
} AviSuperIndexChunk;
    
typedef struct _AviStdIndexChunk {
    AviIndexChunkHeader header;
    AviStdIndexEntry *aIndex;
} AviStdIndexChunk;
    
typedef struct _AviStreamInfo {
    AviStreamHeader strh;
    union {
        BITMAPINFOHEADER video;
        WAVEFORMATEX audio;
    } format;
    
    char *strn;
    AviSuperIndexChunk super_index;
    
    DWORD num_std_index_list;
    AviStdIndexChunk *std_index_list;
} AviStreamInfo;

typedef struct _AviInfo {
    AviMainHeader main_header;

    long num_stream_info;
    AviStreamInfo *stream_infos;
    spBool have_super_index;

    AviOldIndexChunk old_index;

    AviExtHeader ext_header;
} AviInfo;
    
#define WAVE_FORMAT_UNKNOWN_STRING "Microsoft Official Unknown"
#define WAVE_FORMAT_PCM_STRING "Microsoft PCM"
#define WAVE_FORMAT_ADPCM_STRING "Microsoft ADPCM"
#define WAVE_FORMAT_IEEE_FLOAT_STRING "Microsoft IEEE float"
#define WAVE_FORMAT_ALAW_STRING "Microsoft A-law"
#define WAVE_FORMAT_MULAW_STRING "Microsoft U-law"
#define WAVE_FORMAT_OKI_ADPCM_STRING "OKI ADPCM format."
#define WAVE_FORMAT_DIGISTD_STRING "Digistd format."
#define WAVE_FORMAT_DIGIFIX_STRING "Digifix format."

#if defined(MACOS)
#pragma import on
#endif

extern long spReadRiffFourCC(char *fourcc, DWORD *size, FILE *fp);
    
extern const char *spGetWavFormatLabelFromTag(WORD wFormatTag);
extern const char *spGetWavFormatLabel(WAVEHEADER wave_header);
extern long spReadWavHeader(WAVEHEADER *wave_header, FILE *fp); /* check if supported */
extern long spParseWavHeader(WAVEHEADER *wave_header, FILE *fp); /* doesn't check if supported */
extern long spWriteWavHeader(WAVEHEADER wave_header, long length, FILE *fp);

extern long spReadAviHeader(AviMainHeader *avi_header, FILE *fp);
extern spBool _spFreeAviIndexChunk(AviIndexChunk *index_chunk);
#define spFreeAviIndexChunk(p) _spFreeAviIndexChunk((AviIndexChunk *)(p))
extern long spReadAviIndexChunk(AviMainHeader *avi_header, long remain_size,
                                AviIndexChunk *index_chunk, FILE *fp);
extern spBool spIsAviStreamValid(AviStreamHeader *avi_strh);
extern spBool spIsAviStreamVideo(AviStreamHeader *avi_strh);
extern spBool spIsAviStreamAudio(AviStreamHeader *avi_strh);
extern spBool spIsAviStreamSubtitle(AviStreamHeader *avi_strh);
extern long spReadAviStreamAudioFormat(AviStreamHeader *avi_strh, WAVEFORMATEX *format, FILE *fp);
extern long spReadAviStreamVideoFormat(AviStreamHeader *avi_strh, BITMAPINFOHEADER *format, FILE *fp);
extern
spBool spFreeAviStreamInfos(AviMainHeader *avi_header,   /* (i) */
                            AviStreamInfo *stream_infos);/* (t) */
extern
AviStreamInfo *spReadAviStreamInfos( /* (o) */
                                    AviMainHeader *avi_header, /* (i) */
                                    FILE *fp); /* (i) */
extern
DWORD spReadAviStdIndex(AviMainHeader *avi_header,   /* (i) */
                        AviStreamInfo *stream_infos, /* (i/o) */
                        AviOldIndexChunk *old_index, /* (o) */
                        FILE *fp);                   /* (i) */
extern
int spSeekAviChunkByStdIndex(AviMainHeader *avi_header, /* (i) */
                             AviStdIndexChunk *std_index_chunk, /* (i) */
                             DWORD count, /* (i) */
                             FILE *fp);   /* (i) */

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPWAVE_WAV_H */
