/*
 *        wavinfo.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sp/spBaseLib.h>
#include <sp/spWave.h>
#include <sp/spWave_Wav.h>
#include <sp/spMain.h>

#define DEBUG

static spBool help_flag;
static int debug_level = -1;
static char o_raw_file[SP_MAX_PATHNAME] = "";
static long o_stream_id = 0L;

static spOptions options;
static spOption option[] = {
    {"-O", NULL, "output raw video to file", NULL,
         SP_TYPE_STRING_A, o_raw_file, NULL},
    {"-S", NULL, "output stream id (0 based index)", NULL,
         SP_TYPE_LONG, &o_stream_id, "0"},
    {"-debug", NULL, "debug level", NULL,
         SP_TYPE_INT, &debug_level, NULL},
    {"-help", "-h", "display this message", NULL,
         SP_TYPE_BOOLEAN, &help_flag, SP_FALSE_STRING},
};

static const char *filelabel[] = {
    "<filename...>",
};

int spMain(int argc, char *argv[])
{
    const char *filename;
    FILE *fp;
    long file_count;
    long k, l, m;
    long nread;
    long total_read;
    AviMainHeader avi_header;
    AviStreamInfo *stream_infos;
    AviOldIndexChunk old_index;
    AviStdIndexChunk *std_index;
    char chunk_id[5];
    unsigned long chunk_size, dwSize;
    char *buffer;
    long buffer_size;
    FILE *o_raw_fp = NULL;
    
    spSetHelpMessage(&help_flag, "Get AVI file information (sample program)");
    options = spGetOptions(argc, argv, option, filelabel);
    spGetOptionsValue(argc, argv, options);

    spSetDebugLevel(debug_level);
    
    filename = spGetFile(options);
    if (filename == NULL) {
        spError(1, "Not enough files.\n");
    }

    file_count = 0;
    do {
        /* open file */
        if (NULL == (fp = spOpenFile(filename, "rb"))) {
            return 1;
        }
        if (file_count == 0) {
            if (!spStrNone(o_raw_file)) {
                if (NULL == (o_raw_fp = spOpenFile(o_raw_file, "wb"))) {
                    return 1;
                }
            }
        }
        
        printf("%s:\n", filename);
        total_read = 0;
        
        /* read information */
        if ((nread = spReadAviHeader(&avi_header, fp)) <= 0) {
            printf("  ");
            printf("This file is not AVI file.\n");
        } else {
            total_read += nread;
            
            if ((stream_infos = spReadAviStreamInfos(&avi_header, fp)) <= 0) {
                return 1;
            }
            total_read += nread;

            if (spReadAviStdIndex(&avi_header, stream_infos, &old_index, fp) > 0) {
                chunk_id[4] = '\0';

                for (m = 0; m < avi_header.dwStreams; m++) {
                    if (stream_infos[m].num_std_index_list > 0L) {
                        printf("Stream %ld dwSuggestedBufferSize: %ld\n", m, stream_infos[m].strh.dwSuggestedBufferSize);
                        buffer_size = MAX(stream_infos[m].strh.dwSuggestedBufferSize + 2048, 16384);
                        buffer = xspAlloc(buffer_size, char);

                        for (l = 0; l < stream_infos[m].num_std_index_list; l++) {
                            std_index = &stream_infos[m].std_index_list[l];
                            for (k = 0; k < std_index->header.nEntriesInUse; k++) {
                                dwSize = 0x7fffffff & std_index->aIndex[k].dwSize;
#ifdef DEBUG
                                printf("index %ld-%ld: dwOffset = %ld, dwSize = %ld\n", l, k, std_index->aIndex[k].dwOffset, dwSize);
#endif
                
                                spSeekAviChunkByStdIndex(&avi_header, std_index, k, fp);
                                spReadRiffFourCC(chunk_id, &chunk_size, fp);
#ifdef DEBUG
                                printf("index %ld-%ld: chunk_id = %s, chunk_size = %ld\n", l, k, chunk_id, chunk_size);
#endif

                                if (chunk_size != dwSize) {
                                    printf("**** ERROR: chunk_size (%ld) != dwSize (%ld) ****\n", chunk_size, dwSize);
                                }
                                if (chunk_size > buffer_size) {
                                    printf("**** ERROR: chunk_size (%ld) > buffer_size (%ld) ****\n", chunk_size, buffer_size);
                                } else {
                                    fread(buffer, 1, chunk_size, fp);
                                    if (o_raw_fp != NULL && m == o_stream_id) {
                                        fwrite(buffer, 1, chunk_size, o_raw_fp);
                                    }
                                }
                            }
                        }
                
                        xspFree(buffer);
                    }
                }
            }

            spFreeAviStreamInfos(&avi_header, stream_infos);
        }
    
        /* close file */
        spCloseFile(fp);
        if (o_raw_fp != NULL) spCloseFile(o_raw_fp);
        o_raw_fp = NULL;
        
        ++file_count;
    } while ((filename = spGetFile(options)) != NULL);
    
    return 0;
}
